@@ -25,6 +25,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <stdio.h>
+#include <dirent.h>
 
 #include "ipkg_conf.h"
 #include "ipkg_cmd.h"
@@ -191,13 +192,7 @@ static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char **argv)
      pkg_src_t *src;
 
  
-     if (conf->offline_root) {
-	  sprintf_alloc(&lists_dir, "%s/%s", 
-			conf->offline_root, 
-			conf->restrict_to_default_dest ? conf->default_dest->lists_dir : conf->lists_dir);
-     } else {
-	  sprintf_alloc(&lists_dir, "%s", conf->restrict_to_default_dest ? conf->default_dest->lists_dir : conf->lists_dir);
-     }
+    sprintf_alloc(&lists_dir, "%s", conf->restrict_to_default_dest ? conf->default_dest->lists_dir : conf->lists_dir);
  
     if (! file_is_dir(lists_dir)) {
 	  if (file_exists(lists_dir)) {
@@ -336,11 +331,90 @@ int ipkg_multiple_files_scan(ipkg_conf_t *conf, int argc, char **argv)
      return 0;
 }
 
+struct ipkg_intercept
+{
+    char *oldpath;
+    char *statedir;
+};
+
+typedef struct ipkg_intercept *ipkg_intercept_t;
+
+ipkg_intercept_t ipkg_prep_intercepts(ipkg_conf_t *conf)
+{
+    ipkg_intercept_t ctx;
+    char *newpath;
+    int gen;
+
+    ctx = malloc (sizeof (*ctx));
+    ctx->oldpath = strdup (getenv ("PATH"));
+
+    sprintf_alloc (&newpath, "%s/ipkg/intercept:%s", DATADIR, ctx->oldpath);
+    setenv ("PATH", newpath, 1);
+    free (newpath);
+    
+    gen = 0;
+ retry:
+    sprintf_alloc (&ctx->statedir, "/tmp/ipkg-intercept-%d-%d", getpid (), gen);
+    if (mkdir (ctx->statedir, 0770) < 0) {
+	if (errno == EEXIST) {
+	    free (ctx->statedir);
+	    gen++;
+	    goto retry;
+	}
+	perror (ctx->statedir);
+	return NULL;
+    }
+    setenv ("IPKG_INTERCEPT_DIR", ctx->statedir, 1);
+    return ctx;
+}
+
+int ipkg_finalize_intercepts(ipkg_intercept_t ctx)
+{
+    char *cmd;
+    DIR *dir;
+    int err = 0;
+
+    setenv ("PATH", ctx->oldpath, 1);
+    free (ctx->oldpath);
+
+    dir = opendir (ctx->statedir);
+    if (dir) {
+	struct dirent *de;
+	while (de = readdir (dir), de != NULL) {
+	    char *path;
+	    
+	    if (de->d_name[0] == '.')
+		continue;
+	    
+	    sprintf_alloc (&path, "%s/%s", ctx->statedir, de->d_name);
+	    if (access (path, X_OK) == 0) {
+		if (system (path)) {
+		    err = errno;
+		    perror (de->d_name);
+		}
+	    }
+	    free (path);
+	}
+    } else
+	perror (ctx->statedir);
+	
+    sprintf_alloc (&cmd, "rm -rf %s", ctx->statedir);
+    system (cmd);
+    free (cmd);
+
+    free (ctx->statedir);
+    free (ctx);
+
+    return err;
+}
+
 int ipkg_configure_packages(ipkg_conf_t *conf, char *pkg_name)
 {
      pkg_vec_t *all;
      int i;
      pkg_t *pkg;
+     ipkg_intercept_t ic;
+     int r, err = 0;
 
      ipkg_message(conf, IPKG_INFO,
 		  "Configuring unpacked packages\n");
@@ -349,6 +423,8 @@ int ipkg_configure_packages(ipkg_conf_t *conf, char *pkg_name)
      all = pkg_vec_alloc();
      pkg_hash_fetch_available(&conf->pkg_hash, all);
 
+     ic = ipkg_prep_intercepts (conf);
+    
      for(i = 0; i < all->len; i++) {
 	  pkg = all->pkgs[i];
 
@@ -359,16 +435,24 @@ int ipkg_configure_packages(ipkg_conf_t *conf, char *pkg_name)
 	       ipkg_message(conf, IPKG_NOTICE,
 			    "Configuring %s\n", pkg->name);
 	       fflush( stdout );
-	       if (ipkg_configure(conf, pkg) == 0) {
+	       r = ipkg_configure(conf, pkg);
+	       if (r == 0) {
 		    pkg->state_status = SS_INSTALLED;
 		    pkg->parent->state_status = SS_INSTALLED;
 		    pkg->state_flag &= ~SF_PREFER;
+	       } else {
+		    if (!err)
+			err = r;
 	       }
 	  }
      }
 
+     r = ipkg_finalize_intercepts (ic);
+     if (r && !err)
+	 err = r;
+
      pkg_vec_free(all);
-     return 0;
+     return err;
 }
 
 static ipkg_conf_t *global_conf;
