@@ -189,11 +189,7 @@ int ipkg_prepare_url_for_install(ipkg_conf_t *conf, const char *url, char **name
         return 0;
      }
      if (namep) {
-	  // *namep = pkg->name;
 	  *namep = strdup(pkg->name);
-// Are this needed to avoid a mem leak ?
-          pkg_deinit(pkg);
-          free(pkg);
      }
      return 0;
 }
