@@ -20,20 +20,39 @@
 #include <stdarg.h>
 #include "file_util.h"
 #include "str_util.h"
+#ifdef IPKG_LIB	
+#include "libipkg.h"
+#endif	
 
+
+#ifdef IPKG_LIB
+static char *question = NULL;
+static int question_len = 255;
+#endif	
 char *get_user_response(const char *format, ...)
 {
-    va_list ap;
-    char *response;
-    va_start(ap, format);
+     int len = question_len;
+     va_list ap;
+     char *response;
+     va_start(ap, format);
 
-    vprintf(format, ap);
-    do {
-        response = file_read_line_alloc(stdin);
-    } while (response == NULL);
+#ifndef IPKG_LIB
+     vprintf(format, ap);
+     do {
+	  response = file_read_line_alloc(stdin);
+     } while (response == NULL);
+#else
+     do {
+	  if (question == NULL || len > question_len) {
+	       question = realloc(question, len + 1);
+	       question_len = len;
+	  }
+	  len = vsnprintf(question,question_len,format,ap);
+     } while (len > question_len);
+     response = strdup(ipkg_cb_response(question));
+#endif
+     str_chomp(response);
+     str_tolower(response);
 
-    str_chomp(response);
-    str_tolower(response);
-
-    return response;
+     return response;
 }
